@@ -322,6 +322,12 @@ int FileIO::autoread(ProtocolDataMap& pdmap, const STD_string& filename, const F
     ODINLOG(odinlog,normalDebug) << "ExpDuration=" << prot.seqpars.get_ExpDuration() << STD_endl;
     prot.seqpars.set_ExpDuration(STD_max(prot.seqpars.get_ExpDuration(), shape(0)*prot.seqpars.get_RepetitionTime()/(60.0*1000.0)));
 
+    // Set label of protocol to filename
+    STD_string postfix;
+    if(pdmapin.size()>1) postfix=":"+itos(cnt,pdmapin.size()-1);
+    prot.set_label("Protocol of "+filename+postfix);
+
+
     const Data<float,4>& data=pdit->second;
     if(!opts.fmap && data.is_filemapped()) {
       ODINLOG(odinlog,normalDebug) << "filemapped" << STD_endl;
@@ -330,7 +336,9 @@ int FileIO::autoread(ProtocolDataMap& pdmap, const STD_string& filename, const F
       pdmap[prot].reference(data);
     }
 
-    ODINLOG(odinlog,loglevel()) << "Read dataset " << cnt++ << " from file " << filename << "  with dimensions " << shape << STD_endl;
+    ODINLOG(odinlog,loglevel()) << "Read dataset " << cnt << " from file " << filename << "  with dimensions " << shape << STD_endl;
+
+    cnt++;
   }
   ODINLOG(odinlog,normalDebug) << "mem(read): " << Profiler::get_memory_usage() << STD_endl;
 
