@@ -294,7 +294,6 @@ struct DicomFormat : public FileFormat {
 
     //detect pixel datatype and copy pixel data
     //@todo: Transpose data and geometry in plane according to DCM_InPlanePhaseEncodingDirection
-    Uint16 bits;
     if(!img.isMonochrome()){
       ODINLOG(odinlog,errorLog) <<  "only gray value images are supportet"<< STD_endl;
       return -1;
@@ -335,8 +334,9 @@ struct DicomFormat : public FileFormat {
     #ifdef ODIN_DEBUG
     double min,max;
     img.getMinMaxValues(min,max);
-    ODINLOG(odinlog,verboseDebug) << "image has " << bits << " significant bits its range is " << min << " - " << max << STD_endl;
-    #endif //ODIN_DEBUG
+    ODINLOG(odinlog,normalDebug) << "image range is " << min << " - " << max << STD_endl;
+	ODINLOG(odinlog,normalDebug) << "datatype set to " << prot.system.get_data_type() << STD_endl;
+	#endif //ODIN_DEBUG
 
     STD_string unknown="Unknown";
 
@@ -536,7 +536,8 @@ struct DicomFormat : public FileFormat {
 
     // Timestamp temporal sorting of multi-file DICOMs
     prot.seqpars.set_AcquisitionStart(-1.0); // inidicates no useful value
-    if(dset->findAndGetOFString(DCM_AcquisitionTime, valstr).good()){// Try timestamp field first
+	if(dset->findAndGetOFString(DCM_InstanceNumber, valstr).good()) prot.seqpars.set_AcquisitionStart(atof(valstr.c_str()));
+	else if(dset->findAndGetOFString(DCM_AcquisitionTime, valstr).good()){// Try timestamp field first
       //@todo backward compatibility missing (http://idlastro.gsfc.nasa.gov/idl_html_help/Value_Representations.html#TM)
       int hh=atoi(valstr.substr(0,2).c_str());
       int mm=atoi(valstr.substr(2,2).c_str());
@@ -545,9 +546,6 @@ struct DicomFormat : public FileFormat {
       prot.seqpars.set_AcquisitionStart( 1000.0*(ss + 60*mm + 3600*hh) );
     } else {
       if(dset->findAndGetOFString(DCM_TriggerTime, valstr).good()) prot.seqpars.set_AcquisitionStart(atof(valstr.c_str())); // Trigger time works well on GE
-      else {
-        if(dset->findAndGetOFString(DCM_InstanceNumber, valstr).good()) prot.seqpars.set_AcquisitionStart(atof(valstr.c_str())); // Use instance number as fallback
-      }
     }
 
     return shape(0)*shape(1);
