@@ -4,8 +4,10 @@
 #include "filter_align.h"
 #include "filter_clip.h"
 #include "filter_detrend.h"
+#include "filter_lowpass.h"
 #include "filter_mask.h"
 #include "filter_merge.h"
+#include "filter_mip.h"
 #include "filter_nan.h"
 #include "filter_resize.h"
 #include "filter_rot.h"
@@ -39,6 +41,7 @@ void FilterStep::create_templates(STD_list<FilterStep*>& result) {
   result.push_back(new FilterTypeMin());
 
   result.push_back(new FilterDeTrend());
+  result.push_back(new FilterLowPass());
 
   result.push_back(new FilterGenMask());
   result.push_back(new FilterUseMask());
@@ -67,6 +70,8 @@ void FilterStep::create_templates(STD_list<FilterStep*>& result) {
   result.push_back(new FilterMerge());
   result.push_back(new FilterSplice());
 
+  result.push_back(new FilterMip());
+
 }
 
 bool FilterStep::process(Data<float,4>& data, Protocol& prot)const{
