@@ -27,14 +27,22 @@ RecoController::RecoController(JcampDxBlock& parblock) : input(0), pipeline(0),
   timeout.set_cmdline_option("to").set_description("Timeout for reco in seconds, 0=inifinite");
   parblock.append(timeout);
 
+  maxrecursion=100;
+  maxrecursion.set_cmdline_option("mr").set_description("Maximum recursion of reconstruction when waiting for blackboard data, zero means no recursion");
+  parblock.append(maxrecursion);
+
   conjPhaseFT=false;
   conjPhaseFT.set_cmdline_option("cp").set_description("Use direct Fourier transform with conjugate-phase correction instead of gridding");
   parblock.append(conjPhaseFT);
 
   slidingWindow=false;
-  slidingWindow.set_cmdline_option("sw").set_description("Sliding window, and if possible keyhole, reconstruction, e.g. for Spiral/PROPELLER");
+  slidingWindow.set_cmdline_option("sw").set_description("Sliding window reconstruction, e.g. for Spiral/PROPELLER");
   parblock.append(slidingWindow);
 
+  keyhole=false;
+  keyhole.set_cmdline_option("kh").set_description("Keyhole reconstruction, e.g. for PROPELLER");
+  parblock.append(keyhole);
+
   sepChannels=false;
   sepChannels.set_cmdline_option("sc").set_description("Store images from channels separately instead of combining them");
   parblock.append(sepChannels);
@@ -51,6 +59,10 @@ RecoController::RecoController(JcampDxBlock& parblock) : input(0), pipeline(0),
   disableMultiFreq.set_cmdline_option("dm").set_description("Disable fieldmap-based multi-frequency reconstruction");
   parblock.append(disableMultiFreq);
 
+  disableSliceTime=false;
+  disableSliceTime.set_cmdline_option("ds").set_description("Disable slice-time correction");
+  parblock.append(disableSliceTime);
+
   phaseCourse=false;
   phaseCourse.set_cmdline_option("pt").set_description("Combine and store phase timecourse from all channels");
   parblock.append(phaseCourse);
@@ -64,7 +76,7 @@ RecoController::RecoController(JcampDxBlock& parblock) : input(0), pipeline(0),
   parblock.append(postproc3d);
 
   dumpRawPrefix="";
-  dumpRawPrefix.set_cmdline_option("dr").set_description("Dump signal data and its protocol to files with this prefix, do not reconstruct images");
+  dumpRawPrefix.set_cmdline_option("rd").set_description("Dump signal data and its protocol to files with this prefix, do not reconstruct images");
   parblock.append(dumpRawPrefix);
 
   fieldmapFile="";
@@ -115,9 +127,8 @@ TinyVector<float,3> RecoController::kspace_extent() const {
   TinyVector<float,3> result; result=0;
   TinyVector<int,3> imgshape=image_size();
 
-  Protocol prot(protocol());
   for(int i=0; i<n_directions; i++) {
-    float fov=prot.geometry.get_FOV(direction(2-i));
+    float fov=prot_cache.geometry.get_FOV(direction(2-i));
     result(i)=secureDivision(2.0*PII*imgshape(i), fov);
   }
 
@@ -125,15 +136,6 @@ TinyVector<float,3> RecoController::kspace_extent() const {
 }
 
 
-
-Protocol RecoController::protocol() const {
-  Protocol prot;
-  MutexLock lock(inputmutex);
-  if(input) prot=input->protocol();
-  return prot;
-}
-
-
 bool RecoController::inquire_data(const STD_string& label, RecoData& data) {
   Log<Reco> odinlog("RecoController","inquire_data");
 
@@ -144,9 +146,10 @@ bool RecoController::inquire_data(const STD_string& label, RecoData& data) {
   if(blackboard.inquire(label,data)) return true; // 1st shot without feeding pipeline or signalling waiting state
   ODINLOG(odinlog,normalDebug) <<  label << "(" << data.coord().print() << ") not ready" << STD_endl;
 
-  unsigned int depth=thread_depth[Thread::self()];
-  ODINLOG(odinlog,normalDebug) << "depth=" << depth << STD_endl;
-  if(depth<100) { // avoid massive recursion
+  int depth=thread_depth[Thread::self()];
+  if(depth<maxrecursion) { // avoid massive recursion
+
+    ODINLOG(odinlog,normalDebug) << "Recursing in depth=" << depth << STD_endl;
 
     while(feed_pipeline()) { // Continue feeding pipeline until no data is available
       if(blackboard.inquire(label,data)) return true; // Return immediately if inquired data becomes available
@@ -242,7 +245,9 @@ bool RecoController::create_numof(const RecoCoord& mask, unsigned short numof[n_
 
 
   has_traj=false;
-  oversampling_cache=0.0;
+  has_oversampling_cache=false;
+  has_relcenter_cache=false;
+
 
   for(CoordCountMap::const_iterator it=countmap->begin(); it!=countmap->end(); ++it) {
     const RecoCoord& coord=it->first;
@@ -254,7 +259,8 @@ bool RecoController::create_numof(const RecoCoord& mask, unsigned short numof[n_
       has_templcorr_cache[coord.index[templtype]]=true;
       has_navigator_cache[coord.index[navigator]]=true;
       if(coord.kspaceTraj) has_traj=true;
-      oversampling_cache=STD_max(oversampling_cache,coord.overSamplingFactor);
+      if(coord.overSamplingFactor>1.0) has_oversampling_cache=true;
+      if(coord.relCenterPos!=0.5) has_relcenter_cache=true;
     }
   }
 
@@ -315,9 +321,9 @@ RecoStep* RecoController::create_pipeline(const STD_string& rec, const RecoCoord
     // check interface between steps
     RecoCoord coordin=step->input_coord();
     if(!coord.compatible(coordin)) {
-      ODINLOG(odinlog,errorLog) << "Interface mismatch: output";
-      if(laststep) ODINLOG(odinlog,errorLog) << "(" << laststep->label() << ")";
-      ODINLOG(odinlog,errorLog) << "=" << coord.print() << " does not match input(" << step->label() << ")=" << coordin.print() << STD_endl;
+      STD_string lastlabel;
+      if(laststep) lastlabel=laststep->label();
+      ODINLOG(odinlog,errorLog) << "Interface mismatch: output(" << lastlabel << ")=" << coord.print(RecoIndex::all) << " does not match input(" << step->label() << ")=" << coordin.print(RecoIndex::all) << STD_endl;
       return 0;
     }
 
@@ -446,12 +452,18 @@ bool RecoController::autorecipe(const RecoCoord& mask, STD_string& result) const
 
 
   STD_string adcprep;
+  if(!has_traj && has_relcenter_cache) adcprep+="adc_pad | ";
+  
   STD_string adcdeapodize;
+  STD_string adcdeapodize_blade;
   if(has_readoutshape) {
     STD_string gridarg;
     if(has_blade) gridarg="(false)"; // no deapo shift
     adcprep+="adc_gridprep | grid1d"+gridarg+" | "; // regrid first to correct for antisymmetric gradient distortions before reflecting / adc_regrid does not work well for blades yet
-    if(!has_blade && !dumpKspace) adcdeapodize="deapodize1d | ";
+    if(!dumpKspace) {
+      if(has_blade) adcdeapodize_blade="deapodize1d | ";
+      else          adcdeapodize      ="deapodize1d | ";
+    }
   }
   if(has_flag_cache&recoReflectBit) adcprep+="adc_reflect | ";
 
@@ -469,7 +481,7 @@ bool RecoController::autorecipe(const RecoCoord& mask, STD_string& result) const
   }
 
   STD_string oversamplingstr;
-  if(oversampling_cache>1.0 && !dumpKspace) oversamplingstr="oversampling | ";
+  if(has_oversampling_cache && !dumpKspace) oversamplingstr="oversampling | ";
 
   STD_string mfreqbegin;
   STD_string mfreqend;
@@ -521,13 +533,18 @@ bool RecoController::autorecipe(const RecoCoord& mask, STD_string& result) const
       if(reduction_factor3d>1) weightsstr+="grappaweights3d("+redfactstr3d+") | ";
     }
     interpolstr+="chancoll | "+weightsstr;
-    if(reduction_factor>1)   interpolstr+="grappa("  +redfactstr+  ") | ";
-    if(reduction_factor3d>1) interpolstr+="grappa3d("+redfactstr3d+") | ";
+
+    STD_string keephshapestr;
+    if(has_blade) keephshapestr=",true"; // do not expand k-space to image matrix size
+
+    if(reduction_factor>1)   interpolstr+="grappa("  +redfactstr+keephshapestr+  ") | ";
+    if(reduction_factor3d>1) interpolstr+="grappa3d("+redfactstr3d+keephshapestr+") | ";
     interpolstr+="chansplit | ";
   }
 
   STD_string fftstr="fft | ";
   if(!has_traj && !disableHomodyne) {
+    if(has_relcenter_cache) fftstr="homodyneread | ";
     if(partial_fourier) fftstr="homodyne | ";
     if(partial_fourier3d) fftstr="homodyne3d | ";
   }
@@ -541,12 +558,13 @@ bool RecoController::autorecipe(const RecoCoord& mask, STD_string& result) const
 
 
   STD_string swstr;
-  if(slidingWindow) swstr="slidingwindow | ";
+  if(slidingWindow || (keyhole && has_blade) ) swstr="slidingwindow | ";
 
 
   STD_string kspacestr=adcprep + phasecorrstr + "kspace | " + interpolstr;
   STD_string adcbaselinestr;
   STD_string deapodizestr;
+  STD_string gridstr;
   if(has_traj) {
     adcbaselinestr="adc_baseline | ";
     if(conjPhaseFT) {
@@ -556,6 +574,7 @@ bool RecoController::autorecipe(const RecoCoord& mask, STD_string& result) const
     } else {
       kspacestr="grid2d | line3dcoll | "+swstr+"cyclecoll | cyclesum | ";
       if(!dumpKspace) deapodizestr="deapodize2d | ";
+      gridstr="gridcut | ";
     }
     oversamplingstr="";
   }
@@ -564,15 +583,16 @@ bool RecoController::autorecipe(const RecoCoord& mask, STD_string& result) const
   if(has_blade) {
 
     STD_string keyholeopt;
-//    if(slidingWindow) keyholeopt="(true)";
+    if(keyhole) keyholeopt="(true)";
 
     STD_string bladecorrstr="bladecorr | ";
 
-    if(mfreqend!="") bladecorrstr="fft | "+mfreqend+"magn | fft(false) | ";
+    if(mfreqend!="") bladecorrstr="fft | "+mfreqend+"magn | "+adcdeapodize_blade+"fft(false) | ";
     mfreqend=""; // do not use in rest of pipeline
 
     bladestr=bladecorrstr+swstr+"cyclecoll | bladecomb"+keyholeopt+" | grid2d | line3dcoll | ";
     if(!dumpKspace) deapodizestr="deapodize2d | ";
+    gridstr="gridcut | ";
   }
 
   STD_string maskstr;
@@ -580,7 +600,7 @@ bool RecoController::autorecipe(const RecoCoord& mask, STD_string& result) const
 
 
   STD_string slicetimestr;
-  if(numof_cache[repetition]>1 && !dumpKspace) {
+  if(!disableSliceTime && numof_cache[repetition]>1 && numof_cache[slice]>1 && !dumpKspace) {
     slicetimestr="slicetime | ";
   }
 
@@ -611,7 +631,7 @@ bool RecoController::autorecipe(const RecoCoord& mask, STD_string& result) const
   if(numof_cache[dti]>1) pp3dpost+="dti | ";
 
   result = averagestr + adcbaselinestr + mfreqbegin + kspacestr + bladestr + offsetstr
-                     + kspaceprep + fftstr + adcdeapodize + deapodizestr + phasecoursestr + pp3dpre + chanstr
+                     + kspaceprep + fftstr + adcdeapodize + deapodizestr + gridstr + phasecoursestr + pp3dpre + chanstr
                      + freqcomb_and_oversampling + pp3dpost + maskstr
                      + "slicecoll | image | " + slicetimestr + "store";
 
@@ -634,6 +654,14 @@ bool RecoController::init(RecoReaderInterface* reader, JcampDxBlock& opts, int a
   Log<Reco> odinlog("RecoController","init");
 
   input=reader;
+  if(!input) {
+    ODINLOG(odinlog,errorLog) << "no reader provided" << STD_endl;
+    return false;
+  }
+
+  // Initialize default protocol by reader, do this as afirst step
+  prot_cache=input->protocol();
+
 
   bool modify_args=false; // Do not modify command-line args since the same option may be used in different template instantiations
 
@@ -707,6 +735,8 @@ bool RecoController::init(RecoReaderInterface* reader, JcampDxBlock& opts, int a
   delete_countmap_cache(); // Free initial countmap which is requested only once
 
 
+
+
   // get recipe
   if(recipeFile!="") load(recipe,recipeFile);
   if(recipe=="") {
@@ -742,6 +772,13 @@ bool RecoController::init(RecoReaderInterface* reader, JcampDxBlock& opts, int a
   context.printpostfix=printed_numof;
   if(!pipeline->query(context)) return false;
 
+
+  // Finally, modify protocol according to recipe/pipeline
+  if( !(slidingWindow || keyhole) ) {
+    prot_cache.seqpars.set_RepetitionTime( prot_cache.seqpars.get_RepetitionTime() * numof_cache[cycle] );  // adjust TR according to numof cycles
+  }
+
+
   return true;
 }
 
@@ -815,7 +852,11 @@ bool RecoController::feed_pipeline() {
 
   thread_depth[Thread::self()]++;
 
+  // Final preparations for pipeline
   for(int idim=0; idim<n_recoDims; idim++) coord.index[idim].set_numof(numof_cache[idim]);
+  coord.adcstart=0;
+  coord.adcend=adc.size()-1;
+
   RecoData rd(coord);
   rd.data(Rank<1>()).reference(adc);
 
@@ -867,30 +908,33 @@ bool RecoController::prepare_pipeline() {
     if(fmap.autoread(fieldmapFile)<0) return false;
     ComplexData<3> cfmap(float2real(fmap));
 
-    int nslice=numof_cache[slice];
+//    int nslice=numof_cache[slice];
+    bool phase3dmode=(numof_cache[line3d]>1);
 
     RecoData rdfmap;
     RecoFieldMapUser::modify4fieldmap(rdfmap.coord());
     ComplexData<3>& fmapdata=rdfmap.data(Rank<3>());
 
-    if(fmap.extent(0)==nslice) { // assume multi-slice mode
-      ODINLOG(odinlog,infoLog) << "Posting fieldmap in multi-slice mode with nslice=" << nslice << STD_endl;
+    if(phase3dmode) {
+      ODINLOG(odinlog,infoLog) << "Posting 3D fieldmap"<< STD_endl;
+
+      fmapdata.reference(cfmap);
+      post_data(posted_fmap_str, rdfmap);
+
+    } else {  // assume multi-slice mode
+
+      ODINLOG(odinlog,infoLog) << "Posting fieldmap in multi-slice mode with nslice=" << fmap.extent(0) << STD_endl;
 
       Range all=Range::all();
 
       fmapdata.resize(1,fmap.extent(1),fmap.extent(2));
 
-      for(int islice=0; islice<nslice; islice++) {
+      for(int islice=0; islice<fmap.extent(0); islice++) {
         rdfmap.coord().index[slice]=islice;
         fmapdata(0,all,all)=cfmap(islice,all,all);
         post_data(posted_fmap_str, rdfmap);
       }
 
-    } else {
-      ODINLOG(odinlog,infoLog) << "Posting 3D fieldmap"<< STD_endl;
-
-      fmapdata.reference(cfmap);
-      post_data(posted_fmap_str, rdfmap);
     }
 
   }
