@@ -19,7 +19,7 @@ private:
   JDXint    NumOfSamples;
   JDXdouble PulseDur;
   JDXbool   fMRITrigger;
-  JDXbool   ReversePhase3D;
+  JDXbool   UpDownBlade;
   JDXbool   FieldMap;
 
 
@@ -49,6 +49,7 @@ private:
   SeqObjList slicepart_template;
   SeqObjList slicepart_grappa;
   SeqObjList preppart;
+  SeqObjList bladepart;
   SeqDelay   exc2acq;
   SeqDelay   exc2refoc;
   SeqDelay   refoc2acq;
@@ -85,6 +86,9 @@ private:
 
   SeqRotMatrixVector bladerot;
 
+  SeqMagnReset reset;
+
+
   dvector bladeangels;
 
 public:
@@ -111,10 +115,10 @@ void method_pars_init() {
   Blades=8;
   Blades.set_description("Number of PROPELLER blades");
 
-  ShortAxis=true;
+  ShortAxis=false;
   ShortAxis.set_description("Readout direction along short axis of blades");
 
-  BladeOversampling=5.0;
+  BladeOversampling=1.0;
   BladeOversampling.set_minmaxval(0.0,100.0);
   BladeOversampling.set_unit("%").set_description("Oversampling (overlap) in short direction of blade");
 
@@ -156,8 +160,8 @@ void method_pars_init() {
   fMRITrigger=true;
   fMRITrigger.set_description("External triggering");
 
-  ReversePhase3D=false;
-  ReversePhase3D.set_description("Reverse phase encoding order in z direction");
+  UpDownBlade=false;
+  UpDownBlade.set_description("Sample each blade twice with opposite phase-encoding direction");
 
   FieldMap=false;
   FieldMap.set_description("Fieldmap pre-scan for distortion correction");
@@ -169,7 +173,6 @@ void method_pars_init() {
   // register method parameters for user interface, parameter files, etc.
   append_parameter(Blades,"Blades");
   append_parameter(ShortAxis,"ShortAxis");
-  append_parameter(BladeOversampling,"BladeOversampling");
 
   append_parameter(EPIMode,"EPIMode");
   append_parameter(TakeMinEchoTime,"TakeMinEchoTime");
@@ -183,12 +186,14 @@ void method_pars_init() {
   fmapscan.init("fmapscan");
   append_parameter(fmapscan.get_parblock(),"FieldMapPars");
 
-  append_parameter(ReversePhase3D,"ReversePhase3D");
+  append_parameter(UpDownBlade,"UpDownBlade");
 
   append_parameter(PulseDur,"PulseDur");
 
+  append_parameter(fMRITrigger,"fMRITrigger");
+
   if(systemInfo->get_platform()!=numaris_4) {
-    append_parameter(fMRITrigger,"fMRITrigger");
+    append_parameter(BladeOversampling,"BladeOversampling");
     append_parameter(RampMode,"RampMode");
     append_parameter(RampSteepness,"RampSteepness");
     append_parameter(NumOfGradEchoes,"NumOfGradEchoes",noedit);
@@ -225,7 +230,7 @@ void method_seq_init() {
   float extra_slicethick_refoc=STD_min(0.5*slicethick, 0.3*slicegap);
 
   if(geometryInfo->get_Mode()==voxel_3d) {
-    slicethick=0.9*geometryInfo->get_FOV(sliceDirection);
+    slicethick=0.8*geometryInfo->get_FOV(sliceDirection);
     extra_slicethick_refoc=0.2*slicethick;
   }
 
@@ -275,8 +280,10 @@ void method_seq_init() {
 
   int bladewidth=sizeRadial;
 
-  if(Blades>1) bladewidth=int( ( (1.0-0.01*BladeOversampling) * (0.5*PII*sizeRadial/Blades) + 0.01*BladeOversampling*sizeRadial ) +0.5 );
-
+  if(Blades>1) {
+    float min_bladewidth=sizeRadial*tan(0.5*PII/Blades);  // exact solution
+    bladewidth=int( ( (1.0-0.01*BladeOversampling) * min_bladewidth + 0.01*BladeOversampling*sizeRadial ) +0.5 );
+  }
 
   if(ShortAxis) readpts_blade=bladewidth;
   else          pelines_blade=bladewidth;
@@ -352,15 +359,6 @@ void method_seq_init() {
     ivector indexvec3d=pe3dvec.get_indexvec();
     ODINLOG(odinlog,normalDebug) << "indexvec3d=" << indexvec3d << STD_endl;
 
-    if(ReversePhase3D) {
-      fvector gradints3d_copy(gradints3d);
-      ivector indexvec3d_copy(indexvec3d);
-      unsigned int n=gradints3d.size();
-      for(unsigned int i=0; i<n; i++) {
-        gradints3d[i]=gradints3d_copy[n-1-i];
-        indexvec3d[i]=indexvec3d_copy[n-1-i];
-      }
-    }
 
     float strength3d=0.7*systemInfo->get_max_grad();
 
@@ -392,6 +390,12 @@ void method_seq_init() {
     float gradduration_slice=secureDivision(maxabsintegral, blipstrength_slice);
     blip3d_slice=SeqGradVectorPulse("blip3d_slice", sliceDirection, blipstrength_slice, blipintegrals/maxabsintegral, gradduration_slice);
 
+/*
+    if(UpDownBlade) {
+      deph3d.invert_strength();
+      blip3d_slice.invert_strength();
+    }
+*/
 
     blip3d=SeqGradChanParallel("blip3d");
     blip3d_template=SeqGradChanParallel("blip3d_template");
@@ -412,12 +416,20 @@ void method_seq_init() {
 
   bladerot=SeqRotMatrixVector("bladerot");
 
-  bladeangels.resize(Blades);
-  for(int iblade=0; iblade<Blades; iblade++) {
-    RotMatrix rm("rotmatrix"+itos(iblade));
+  bool full_cycle=UpDownBlade; //(geometryInfo->get_Mode()!=voxel_3d && UpDownBlade);
+
+  int nangles=Blades;
+  if(full_cycle) nangles=2*Blades;
+
+  bladeangels.resize(nangles);
+  for(int iangle=0; iangle<nangles; iangle++) {
+    RotMatrix rm("rotmatrix"+itos(iangle));
+    int iblade=iangle;
+    if(full_cycle) iblade=iangle/2;
     float ang=float(iblade)/float(Blades)*PII; // 0...180 deg
+    if(full_cycle && iangle%2) ang+=PII; // adjacent blades have opposite direction
     rm.set_inplane_rotation(ang);
-    bladeangels[iblade]=ang;
+    bladeangels[iangle]=ang;
     bladerot.append(rm);
   }
 
@@ -487,10 +499,11 @@ void method_seq_init() {
   crusherdelay=SeqDelay("crusherdelay",0.1); // Small delay to avoid gradient-induced stimulation
 
 
-  //////////////// fMRI trigger: //////////////////////////////
+  //////////////// trigger: //////////////////////////////
 
   trigger=SeqTrigger("fmri_trigger",1.0);
 
+  reset=SeqMagnReset("reset");
 
   //////////////// Field-map template: //////////////////////////////
 
@@ -502,6 +515,8 @@ void method_seq_init() {
 
   //////////////// Build the sequence: //////////////////////////////
 
+  preppart+=reset;
+
   epiacq_part=SeqObjList("epiacq_part");
   epiacq_dummy_part=SeqObjList("epiacq_dummy_part");
   epiacq_template_part=SeqObjList("epiacq_template_part");
@@ -531,7 +546,7 @@ void method_seq_init() {
   // add fat saturation to template and repetitions
   if(FatSaturation) preppart += fatsat;
 
-  if(fMRITrigger) slicepart += trigger;
+  if(fMRITrigger && (geometryInfo->get_Mode()!=voxel_3d) ) slicepart+= trigger; // trigger for each single blade
 
   if (EPIMode=="FIDMode") {
 
@@ -608,11 +623,14 @@ void method_seq_init() {
            )[deph_grappa.get_epi_reduction_vector()];
   }
 
+  if(fMRITrigger && (geometryInfo->get_Mode()==voxel_3d) ) bladepart+= trigger; // trigger only for each full blade cycle
 
-  scan+= reploop(
-           bladeloop(
-             slicepart
-           )[bladerot]
+  bladepart+= bladeloop(
+                slicepart
+              )[bladerot];
+
+  scan+= reploop( 
+           bladepart
          )[commonPars->get_NumOfRepetitions()];
 
 
@@ -726,6 +744,8 @@ void method_pars_set() {
   epiacq_grappa.set_reco_vector(cycle,bladerot);
   recoInfo->set_DimValues(cycle,bladeangels);
 
+//  if(geometryInfo->get_Mode()==voxel_3d && UpDownBlade) recoInfo->set_CmdLineOpts("-ip -sflip");
+
 }
 
 };
